@@ -1,22 +1,22 @@
 /*
 	Audio File Library
-	Copyright (C) 1998-2000, 2003, Michael Pruett <michael@68k.org>
+	Copyright (C) 1998-2000, 2003-2004, 2010-2013, Michael Pruett <michael@68k.org>
 	Copyright (C) 2000-2001, Silicon Graphics, Inc.
 
 	This library is free software; you can redistribute it and/or
-	modify it under the terms of the GNU Library General Public
+	modify it under the terms of the GNU Lesser General Public
 	License as published by the Free Software Foundation; either
-	version 2 of the License, or (at your option) any later version.
+	version 2.1 of the License, or (at your option) any later version.
 
 	This library is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	Library General Public License for more details.
+	Lesser General Public License for more details.
 
-	You should have received a copy of the GNU Library General Public
+	You should have received a copy of the GNU Lesser General Public
 	License along with this library; if not, write to the
-	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-	Boston, MA  02111-1307  USA.
+	Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+	Boston, MA  02110-1301  USA
 */
 
 /*
@@ -31,7 +31,6 @@
 
 #include <assert.h>
 #include <stdint.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -61,10 +60,11 @@ const InstParamInfo _af_aiff_inst_params[_AF_AIFF_NUM_INSTPARAMS] =
 const int _af_aiffc_compression_types[_AF_AIFFC_NUM_COMPTYPES] =
 {
 	AF_COMPRESSION_G711_ULAW,
-	AF_COMPRESSION_G711_ALAW
+	AF_COMPRESSION_G711_ALAW,
+	AF_COMPRESSION_IMA
 };
 
-static _AFfilesetup aiff_default_filesetup =
+static const _AFfilesetup aiffDefaultFileSetup =
 {
 	_AF_VALID_FILESETUP,	/* valid */
 	AF_FILE_AIFF,		/* fileFormat */
@@ -102,13 +102,13 @@ AIFFFile::AIFFFile()
 {
 	setFormatByteOrder(AF_BYTEORDER_BIGENDIAN);
 
-	miscellaneousPosition = 0;
-	FVER_offset = 0;
-	COMM_offset = 0;
-	MARK_offset = 0;
-	INST_offset = 0;
-	AESD_offset = 0;
-	SSND_offset = 0;
+	m_miscellaneousPosition = 0;
+	m_FVER_offset = 0;
+	m_COMM_offset = 0;
+	m_MARK_offset = 0;
+	m_INST_offset = 0;
+	m_AESD_offset = 0;
+	m_SSND_offset = 0;
 }
 
 /*
@@ -143,7 +143,7 @@ status AIFFFile::parseAESD(const Tag &type, size_t size)
 		Try to read 24 bytes of AES nonaudio data from the file.
 		Fail if the file disappoints.
 	*/
-	if (fh->read(aesChannelStatusData, 24) != 24)
+	if (m_fh->read(aesChannelStatusData, 24) != 24)
 		return AF_FAIL;
 
 	memcpy(track->aesData, aesChannelStatusData, 24);
@@ -170,9 +170,9 @@ status AIFFFile::parseMiscellaneous(const Tag &type, size_t size)
 	if (size == 0)
 		return AF_FAIL;
 
-	miscellaneousCount++;
-	miscellaneous = (Miscellaneous *) _af_realloc(miscellaneous,
-		miscellaneousCount * sizeof (Miscellaneous));
+	m_miscellaneousCount++;
+	m_miscellaneous = (Miscellaneous *) _af_realloc(m_miscellaneous,
+		m_miscellaneousCount * sizeof (Miscellaneous));
 
 	if (type == "NAME")
 		misctype = AF_MISC_NAME;
@@ -187,12 +187,12 @@ status AIFFFile::parseMiscellaneous(const Tag &type, size_t size)
 	else if (type == "MIDI")
 		misctype = AF_MISC_MIDI;
 
-	miscellaneous[miscellaneousCount - 1].id = miscellaneousCount;
-	miscellaneous[miscellaneousCount - 1].type = misctype;
-	miscellaneous[miscellaneousCount - 1].size = size;
-	miscellaneous[miscellaneousCount - 1].position = 0;
-	miscellaneous[miscellaneousCount - 1].buffer = _af_malloc(size);
-	fh->read(miscellaneous[miscellaneousCount - 1].buffer, size);
+	m_miscellaneous[m_miscellaneousCount - 1].id = m_miscellaneousCount;
+	m_miscellaneous[m_miscellaneousCount - 1].type = misctype;
+	m_miscellaneous[m_miscellaneousCount - 1].size = size;
+	m_miscellaneous[m_miscellaneousCount - 1].position = 0;
+	m_miscellaneous[m_miscellaneousCount - 1].buffer = _af_malloc(size);
+	m_fh->read(m_miscellaneous[m_miscellaneousCount - 1].buffer, size);
 
 	return AF_SUCCEED;
 }
@@ -217,8 +217,8 @@ status AIFFFile::parseINST(const Tag &type, size_t size)
 	instrument->loopCount = 2;
 	instrument->loops = (Loop *) _af_calloc(2, sizeof (Loop));
 
-	instrumentCount = 1;
-	instruments = instrument;
+	m_instrumentCount = 1;
+	m_instruments = instrument;
 
 	readU8(&baseNote);
 	readS8(&detune);
@@ -228,13 +228,6 @@ status AIFFFile::parseINST(const Tag &type, size_t size)
 	readU8(&highVelocity);
 	readS16(&gain);
 
-#ifdef DEBUG
-	printf("baseNote/detune/lowNote/highNote/lowVelocity/highVelocity/gain:"
-		" %d %d %d %d %d %d %d\n",
-		baseNote, detune, lowNote, highNote, lowVelocity, highVelocity,
-		gain);
-#endif
-
 	instrument->values[0].l = baseNote;
 	instrument->values[1].l = detune;
 	instrument->values[2].l = lowVelocity;
@@ -254,14 +247,6 @@ status AIFFFile::parseINST(const Tag &type, size_t size)
 	readU16(&releaseLoopBegin);
 	readU16(&releaseLoopEnd);
 
-#ifdef DEBUG
-	printf("sustain loop: mode %d, begin %d, end %d\n",
-		sustainLoopPlayMode, sustainLoopBegin, sustainLoopEnd);
-
-	printf("release loop: mode %d, begin %d, end %d\n",
-		releaseLoopPlayMode, releaseLoopBegin, releaseLoopEnd);
-#endif
-
 	instrument->loops[0].id = 1;
 	instrument->loops[0].mode = sustainLoopPlayMode;
 	instrument->loops[0].beginMarker = sustainLoopBegin;
@@ -302,19 +287,12 @@ status AIFFFile::parseMARK(const Tag &type, size_t size)
 
 		readU16(&markerID);
 		readU32(&markerPosition);
-		fh->read(&sizeByte, 1);
+		m_fh->read(&sizeByte, 1);
 		markerName = (char *) _af_malloc(sizeByte + 1);
-		fh->read(markerName, sizeByte);
+		m_fh->read(markerName, sizeByte);
 
 		markerName[sizeByte] = '\0';
 
-#ifdef DEBUG
-		printf("marker id: %d, position: %d, name: %s\n",
-			markerID, markerPosition, markerName);
-
-		printf("size byte: %d\n", sizeByte);
-#endif
-
 		/*
 			If sizeByte is even, then 1+sizeByte (the length
 			of the string) is odd.	Skip an extra byte to
@@ -322,7 +300,7 @@ status AIFFFile::parseMARK(const Tag &type, size_t size)
 		*/
 
 		if ((sizeByte % 2) == 0)
-			fh->seek(1, File::SeekFromCurrent);
+			m_fh->seek(1, File::SeekFromCurrent);
 
 		track->markers[i].id = markerID;
 		track->markers[i].position = markerPosition;
@@ -352,29 +330,37 @@ status AIFFFile::parseCOMM(const Tag &type, size_t size)
 	readU16(&numChannels);
 	track->f.channelCount = numChannels;
 
+	if (!numChannels)
+	{
+		_af_error(AF_BAD_CHANNELS, "invalid file with 0 channels");
+		return AF_FAIL;
+	}
+
 	readU32(&numSampleFrames);
 	track->totalfframes = numSampleFrames;
 
 	readU16(&sampleSize);
 	track->f.sampleWidth = sampleSize;
 
-	fh->read(sampleRate, 10);
+	m_fh->read(sampleRate, 10);
 	track->f.sampleRate = _af_convert_from_ieee_extended(sampleRate);
 
 	track->f.compressionType = AF_COMPRESSION_NONE;
 	track->f.sampleFormat = AF_SAMPFMT_TWOSCOMP;
 	track->f.byteOrder = AF_BYTEORDER_BIGENDIAN;
 
-	if (fileFormat == AF_FILE_AIFFC)
+	track->f.framesPerPacket = 1;
+
+	if (isAIFFC())
 	{
 		Tag compressionID;
-		/* Pascal strings are at most 255 bytes long. */
+		// Pascal strings are at most 255 bytes long.
 		char compressionName[256];
 
 		readTag(&compressionID);
 
-		/* Read the Pascal-style string containing the name. */
-		af_read_pstring(compressionName, fh);
+		// Read the Pascal-style string containing the name.
+		readPString(compressionName);
 
 		if (compressionID == "NONE" || compressionID == "twos")
 		{
@@ -401,10 +387,16 @@ status AIFFFile::parseCOMM(const Tag &type, size_t size)
 		else if (compressionID == "ulaw" || compressionID == "ULAW")
 		{
 			track->f.compressionType = AF_COMPRESSION_G711_ULAW;
+			track->f.byteOrder = _AF_BYTEORDER_NATIVE;
+			track->f.sampleWidth = 16;
+			track->f.bytesPerPacket = track->f.channelCount;
 		}
 		else if (compressionID == "alaw" || compressionID == "ALAW")
 		{
 			track->f.compressionType = AF_COMPRESSION_G711_ALAW;
+			track->f.byteOrder = _AF_BYTEORDER_NATIVE;
+			track->f.sampleWidth = 16;
+			track->f.bytesPerPacket = track->f.channelCount;
 		}
 		else if (compressionID == "fl32" || compressionID == "FL32")
 		{
@@ -423,6 +415,17 @@ status AIFFFile::parseCOMM(const Tag &type, size_t size)
 			track->f.compressionType = AF_COMPRESSION_NONE;
 			track->f.byteOrder = AF_BYTEORDER_LITTLEENDIAN;
 		}
+		else if (compressionID == "ima4")
+		{
+			track->f.sampleWidth = 16;
+			track->f.sampleFormat = AF_SAMPFMT_TWOSCOMP;
+			track->f.compressionType = AF_COMPRESSION_IMA;
+			track->f.byteOrder = _AF_BYTEORDER_NATIVE;
+
+			initIMACompressionParams();
+
+			track->totalfframes *= 64;
+		}
 		else
 		{
 			_af_error(AF_BAD_NOT_IMPLEMENTED, "AIFF-C compression type '%s' not currently supported",
@@ -431,7 +434,11 @@ status AIFFFile::parseCOMM(const Tag &type, size_t size)
 		}
 	}
 
-	_af_set_sample_format(&track->f, track->f.sampleFormat, track->f.sampleWidth);
+	if (track->f.isUncompressed())
+		track->f.computeBytesPerPacketPCM();
+
+	if (_af_set_sample_format(&track->f, track->f.sampleFormat, track->f.sampleWidth) == AF_FAIL)
+		return AF_FAIL;
 
 	return AF_SUCCEED;
 }
@@ -450,24 +457,9 @@ status AIFFFile::parseSSND(const Tag &type, size_t size)
 	readU32(&offset);
 	readU32(&blockSize);
 
-	/*
-		This seems like a reasonable way to calculate the number of
-		bytes in an SSND chunk.
-	*/
 	track->data_size = size - 8 - offset;
 
-#ifdef DEBUG
-	printf("offset: %d\n", offset);
-	printf("block size: %d\n", blockSize);
-#endif
-
-	track->fpos_first_frame = fh->tell() + offset;
-
-#ifdef DEBUG
-	printf("data start: %d\n", track->fpos_first_frame);
-#endif
-
-	/* Sound data follows. */
+	track->fpos_first_frame = m_fh->tell() + offset;
 
 	return AF_SUCCEED;
 }
@@ -479,24 +471,17 @@ status AIFFFile::readInit(AFfilesetup setup)
 	bool hasCOMM = false;
 	bool hasFVER = false;
 	bool hasSSND = false;
-	bool hasMARK = false;
-	bool hasINST = false;
-	bool hasAESD = false;
 
-	fh->seek(0, File::SeekFromBeginning);
+	m_fh->seek(0, File::SeekFromBeginning);
 
-	fh->read(&type, 4);
+	m_fh->read(&type, 4);
 	readU32(&size);
-	fh->read(&formtype, 4);
+	m_fh->read(&formtype, 4);
 
 	if (memcmp(&type, "FORM", 4) != 0 ||
 		(memcmp(&formtype, "AIFF", 4) && memcmp(&formtype, "AIFC", 4)))
 		return AF_FAIL;
 
-#ifdef DEBUG
-	printf("size: %d\n", size);
-#endif
-
 	if (!allocateTrack())
 		return AF_FAIL;
 
@@ -508,17 +493,9 @@ status AIFFFile::readInit(AFfilesetup setup)
 		uint32_t chunksize = 0;
 		status result = AF_SUCCEED;
 
-#ifdef DEBUG
-		printf("index: %d\n", index);
-#endif
 		readTag(&chunkid);
 		readU32(&chunksize);
 
-#ifdef DEBUG
-		printf("%s", chunkid.name().c_str());
-		printf(" size: %d\n", chunksize);
-#endif
-
 		if (chunkid == "COMM")
 		{
 			hasCOMM = true;
@@ -531,17 +508,14 @@ status AIFFFile::readInit(AFfilesetup setup)
 		}
 		else if (chunkid == "INST")
 		{
-			hasINST = true;
 			parseINST(chunkid, chunksize);
 		}
 		else if (chunkid == "MARK")
 		{
-			hasMARK = true;
 			parseMARK(chunkid, chunksize);
 		}
 		else if (chunkid == "AESD")
 		{
-			hasAESD = true;
 			parseAESD(chunkid, chunksize);
 		}
 		else if (chunkid == "NAME" ||
@@ -577,7 +551,7 @@ status AIFFFile::readInit(AFfilesetup setup)
 		if ((index % 2) != 0)
 			index++;
 
-		fh->seek(index + 8, File::SeekFromBeginning);
+		m_fh->seek(index + 8, File::SeekFromBeginning);
 	}
 
 	if (!hasCOMM)
@@ -585,6 +559,11 @@ status AIFFFile::readInit(AFfilesetup setup)
 		_af_error(AF_BAD_AIFF_COMM, "bad AIFF COMM chunk");
 	}
 
+	if (isAIFFC() && !hasFVER)
+	{
+		_af_error(AF_BAD_HEADER, "FVER chunk is required in AIFF-C");
+	}
+
 	/* The file has been successfully parsed. */
 	return AF_SUCCEED;
 }
@@ -665,16 +644,27 @@ AFfilesetup AIFFFile::completeSetup(AFfilesetup setup)
 		return AF_NULL_FILESETUP;
 	}
 
-	/* XXXmpruett handle compression here */
+	if (track->f.compressionType != AF_COMPRESSION_NONE &&
+		track->f.compressionType != AF_COMPRESSION_G711_ULAW &&
+		track->f.compressionType != AF_COMPRESSION_G711_ALAW &&
+		track->f.compressionType != AF_COMPRESSION_IMA)
+	{
+		_af_error(AF_BAD_NOT_IMPLEMENTED, "compression format not supported in AIFF-C");
+		return AF_NULL_FILESETUP;
+	}
 
-	if (track->byteOrderSet &&
+	if (track->f.isUncompressed() &&
+		track->byteOrderSet &&
 		track->f.byteOrder != AF_BYTEORDER_BIGENDIAN &&
-		track->f.sampleWidth > 8)
+		track->f.isByteOrderSignificant())
 	{
 		_af_error(AF_BAD_BYTEORDER,
 			"AIFF/AIFF-C format supports only big-endian data");
+		return AF_NULL_FILESETUP;
 	}
-	track->f.byteOrder = AF_BYTEORDER_BIGENDIAN;
+
+	if (track->f.isUncompressed())
+		track->f.byteOrder = AF_BYTEORDER_BIGENDIAN;
 
 	if (setup->instrumentSet)
 	{
@@ -712,18 +702,19 @@ AFfilesetup AIFFFile::completeSetup(AFfilesetup setup)
 		}
 	}
 
-	return _af_filesetup_copy(setup, &aiff_default_filesetup, true);
+	return _af_filesetup_copy(setup, &aiffDefaultFileSetup, true);
 }
 
 bool AIFFFile::isInstrumentParameterValid(AUpvlist list, int i)
 {
-	int param, type, lval;
+	int param, type;
 
 	AUpvgetparam(list, i, &param);
 	AUpvgetvaltype(list, i, &type);
 	if (type != AU_PVTYPE_LONG)
 		return false;
 
+	long lval;
 	AUpvgetval(list, i, &lval);
 
 	switch (param)
@@ -761,30 +752,28 @@ bool AIFFFile::isInstrumentParameterValid(AUpvlist list, int i)
 
 int AIFFFile::getVersion()
 {
-	if (fileFormat == AF_FILE_AIFFC)
+	if (isAIFFC())
 		return AIFC_VERSION_1;
 	return 0;
 }
 
 status AIFFFile::writeInit(AFfilesetup setup)
 {
-	uint32_t fileSize = 0;
-
-	assert(fileFormat == AF_FILE_AIFF ||
-		fileFormat == AF_FILE_AIFFC);
-
-	if (_af_filesetup_make_handle(setup, this) == AF_FAIL)
+	if (initFromSetup(setup) == AF_FAIL)
 		return AF_FAIL;
 
-	fh->write("FORM", 4);
+	initCompressionParams();
+
+	uint32_t fileSize = 0;
+	m_fh->write("FORM", 4);
 	writeU32(&fileSize);
 
-	if (fileFormat == AF_FILE_AIFF)
-		fh->write("AIFF", 4);
-	else if (fileFormat == AF_FILE_AIFFC)
-		fh->write("AIFC", 4);
+	if (isAIFFC())
+		m_fh->write("AIFC", 4);
+	else
+		m_fh->write("AIFF", 4);
 
-	if (fileFormat == AF_FILE_AIFFC)
+	if (isAIFFC())
 		writeFVER();
 
 	writeCOMM();
@@ -799,19 +788,15 @@ status AIFFFile::writeInit(AFfilesetup setup)
 
 status AIFFFile::update()
 {
-#ifdef DEBUG
-	printf("_af_aiff_update called.\n");
-#endif
-
 	/* Get the length of the file. */
-	uint32_t length = fh->length();
+	uint32_t length = m_fh->length();
 	length -= 8;
 
 	/* Set the length of the FORM chunk. */
-	fh->seek(4, File::SeekFromBeginning);
+	m_fh->seek(4, File::SeekFromBeginning);
 	writeU32(&length);
 
-	if (fileFormat == AF_FILE_AIFFC)
+	if (isAIFFC())
 		writeFVER();
 
 	writeCOMM();
@@ -826,16 +811,14 @@ status AIFFFile::update()
 
 status AIFFFile::writeCOMM()
 {
-	bool isAIFFC = fileFormat == AF_FILE_AIFFC;
-
 	/*
 		If COMM_offset hasn't been set yet, set it to the
 		current offset.
 	*/
-	if (COMM_offset == 0)
-		COMM_offset = fh->tell();
+	if (m_COMM_offset == 0)
+		m_COMM_offset = m_fh->tell();
 	else
-		fh->seek(COMM_offset, File::SeekFromBeginning);
+		m_fh->seek(m_COMM_offset, File::SeekFromBeginning);
 
 	Track *track = getTrack();
 
@@ -843,7 +826,7 @@ status AIFFFile::writeCOMM()
 	/* Pascal strings can occupy only 255 bytes (+ a size byte). */
 	char compressionName[256];
 
-	if (isAIFFC)
+	if (isAIFFC())
 	{
 		if (track->f.compressionType == AF_COMPRESSION_NONE)
 		{
@@ -886,9 +869,14 @@ status AIFFFile::writeCOMM()
 			compressionTag = "alaw";
 			strcpy(compressionName, "CCITT G.711 A-law");
 		}
+		else if (track->f.compressionType == AF_COMPRESSION_IMA)
+		{
+			compressionTag = "ima4";
+			strcpy(compressionName, "IMA 4:1 compression");
+		}
 	}
 
-	fh->write("COMM", 4);
+	m_fh->write("COMM", 4);
 
 	/*
 		For AIFF-C files, the length of the COMM chunk is 22
@@ -898,7 +886,7 @@ status AIFFFile::writeCOMM()
 		include the pad byte in the chunk's size.
 	*/
 	uint32_t chunkSize;
-	if (isAIFFC)
+	if (isAIFFC())
 		chunkSize = 22 + strlen(compressionName) + 1;
 	else
 		chunkSize = 18;
@@ -910,6 +898,8 @@ status AIFFFile::writeCOMM()
 
 	/* number of sample frames, 4 bytes */
 	uint32_t frameCount = track->totalfframes;
+	if (track->f.compressionType == AF_COMPRESSION_IMA)
+		frameCount = track->totalfframes / track->f.framesPerPacket;
 	writeU32(&frameCount);
 
 	/* sample size, 2 bytes */
@@ -919,12 +909,12 @@ status AIFFFile::writeCOMM()
 	/* sample rate, 10 bytes */
 	uint8_t sampleRate[10];
 	_af_convert_to_ieee_extended(track->f.sampleRate, sampleRate);
-	fh->write(sampleRate, 10);
+	m_fh->write(sampleRate, 10);
 
-	if (isAIFFC)
+	if (isAIFFC())
 	{
 		writeTag(&compressionTag);
-		af_write_pstring(compressionName, fh);
+		writePString(compressionName);
 	}
 
 	return AF_SUCCEED;
@@ -941,19 +931,19 @@ status AIFFFile::writeAESD()
 	if (!track->hasAESData)
 		return AF_SUCCEED;
 
-	if (AESD_offset == 0)
-		AESD_offset = fh->tell();
+	if (m_AESD_offset == 0)
+		m_AESD_offset = m_fh->tell();
 	else
-		fh->seek(AESD_offset, File::SeekFromBeginning);
+		m_fh->seek(m_AESD_offset, File::SeekFromBeginning);
 
-	if (fh->write("AESD", 4) < 4)
+	if (m_fh->write("AESD", 4) < 4)
 		return AF_FAIL;
 
 	uint32_t size = 24;
 	if (!writeU32(&size))
 		return AF_FAIL;
 
-	if (fh->write(track->aesData, 24) < 24)
+	if (m_fh->write(track->aesData, 24) < 24)
 		return AF_FAIL;
 
 	return AF_SUCCEED;
@@ -963,12 +953,12 @@ status AIFFFile::writeSSND()
 {
 	Track *track = getTrack();
 
-	if (SSND_offset == 0)
-		SSND_offset = fh->tell();
+	if (m_SSND_offset == 0)
+		m_SSND_offset = m_fh->tell();
 	else
-		fh->seek(SSND_offset, File::SeekFromBeginning);
+		m_fh->seek(m_SSND_offset, File::SeekFromBeginning);
 
-	fh->write("SSND", 4);
+	m_fh->write("SSND", 4);
 
 	uint32_t chunkSize = track->data_size + 8;
 	writeU32(&chunkSize);
@@ -980,7 +970,7 @@ status AIFFFile::writeSSND()
 	writeU32(&zero);
 
 	if (track->fpos_first_frame == 0)
-		track->fpos_first_frame = fh->tell();
+		track->fpos_first_frame = m_fh->tell();
 
 	return AF_SUCCEED;
 }
@@ -1005,7 +995,7 @@ status AIFFFile::writeINST()
 	instrumentdata.releaseLoopEnd =
 		afGetLoopEnd(this, AF_DEFAULT_INST, 2);
 
-	fh->write("INST", 4);
+	m_fh->write("INST", 4);
 	writeU32(&length);
 
 	instrumentdata.baseNote =
@@ -1044,58 +1034,48 @@ status AIFFFile::writeINST()
 
 status AIFFFile::writeMARK()
 {
-	uint16_t numMarkers = afGetMarkIDs(this, AF_DEFAULT_TRACK, NULL);
-	if (numMarkers == 0)
+	Track *track = getTrack();
+	if (!track->markerCount)
 		return AF_SUCCEED;
 
-	if (MARK_offset == 0)
-		MARK_offset = fh->tell();
+	if (m_MARK_offset == 0)
+		m_MARK_offset = m_fh->tell();
 	else
-		fh->seek(MARK_offset, File::SeekFromBeginning);
+		m_fh->seek(m_MARK_offset, File::SeekFromBeginning);
 
-	AFfileoffset chunkStartPosition, chunkEndPosition;
+	Tag markTag("MARK");
 	uint32_t length = 0;
 
-	fh->write("MARK", 4);
+	writeTag(&markTag);
 	writeU32(&length);
 
-	chunkStartPosition = fh->tell();
-
-	int *markids = (int *) _af_calloc(numMarkers, sizeof (int));
-	assert(markids);
-	afGetMarkIDs(this, AF_DEFAULT_TRACK, markids);
+	AFfileoffset chunkStartPosition = m_fh->tell();
 
+	uint16_t numMarkers = track->markerCount;
 	writeU16(&numMarkers);
 
 	for (unsigned i=0; i<numMarkers; i++)
 	{
-		uint16_t id = markids[i];
+		uint16_t id = track->markers[i].id;
 		writeU16(&id);
 
-		uint32_t position = afGetMarkPosition(this, AF_DEFAULT_TRACK, markids[i]);
+		uint32_t position = track->markers[i].position;
 		writeU32(&position);
 
-		const char *name = afGetMarkName(this, AF_DEFAULT_TRACK, markids[i]);
+		const char *name = track->markers[i].name;
 		assert(name);
 
-		/* Write the name as a Pascal-style string. */
-		af_write_pstring(name, fh);
+		// Write the name as a Pascal-style string.
+		writePString(name);
 	}
 
-	free(markids);
-
-	chunkEndPosition = fh->tell();
+	AFfileoffset chunkEndPosition = m_fh->tell();
 	length = chunkEndPosition - chunkStartPosition;
 
-#ifdef DEBUG
-	printf(" end: %d\n", chunkEndPosition);
-	printf(" length: %d\n", length);
-#endif
-
-	fh->seek(chunkStartPosition - 4, File::SeekFromBeginning);
+	m_fh->seek(chunkStartPosition - 4, File::SeekFromBeginning);
 
 	writeU32(&length);
-	fh->seek(chunkEndPosition, File::SeekFromBeginning);
+	m_fh->seek(chunkEndPosition, File::SeekFromBeginning);
 
 	return AF_SUCCEED;
 }
@@ -1107,14 +1087,14 @@ status AIFFFile::writeFVER()
 {
 	uint32_t chunkSize, timeStamp;
 
-	assert(fileFormat == AF_FILE_AIFFC);
+	assert(isAIFFC());
 
-	if (FVER_offset == 0)
-		FVER_offset = fh->tell();
+	if (m_FVER_offset == 0)
+		m_FVER_offset = m_fh->tell();
 	else
-		fh->seek(FVER_offset, File::SeekFromBeginning);
+		m_fh->seek(m_FVER_offset, File::SeekFromBeginning);
 
-	fh->write("FVER", 4);
+	m_fh->write("FVER", 4);
 
 	chunkSize = 4;
 	writeU32(&chunkSize);
@@ -1131,22 +1111,18 @@ status AIFFFile::writeFVER()
 */
 status AIFFFile::writeMiscellaneous()
 {
-	if (miscellaneousPosition == 0)
-		miscellaneousPosition = fh->tell();
+	if (m_miscellaneousPosition == 0)
+		m_miscellaneousPosition = m_fh->tell();
 	else
-		fh->seek(miscellaneousPosition, File::SeekFromBeginning);
+		m_fh->seek(m_miscellaneousPosition, File::SeekFromBeginning);
 
-	for (int i=0; i<miscellaneousCount; i++)
+	for (int i=0; i<m_miscellaneousCount; i++)
 	{
-		Miscellaneous *misc = &miscellaneous[i];
+		Miscellaneous *misc = &m_miscellaneous[i];
 		Tag chunkType;
 		uint32_t chunkSize;
 		uint8_t padByte = 0;
 
-#ifdef DEBUG
-		printf("WriteMiscellaneous: %d, type %d\n", i, misc->type);
-#endif
-
 		switch (misc->type)
 		{
 			case AF_MISC_NAME:
@@ -1173,9 +1149,9 @@ status AIFFFile::writeMiscellaneous()
 			for now.
 		*/
 		if (misc->buffer != NULL)
-			fh->write(misc->buffer, misc->size);
+			m_fh->write(misc->buffer, misc->size);
 		else
-			fh->seek(misc->size, File::SeekFromCurrent);
+			m_fh->seek(misc->size, File::SeekFromCurrent);
 
 		if (misc->size % 2 != 0)
 			writeU8(&padByte);
@@ -1183,3 +1159,62 @@ status AIFFFile::writeMiscellaneous()
 
 	return AF_SUCCEED;
 }
+
+void AIFFFile::initCompressionParams()
+{
+	Track *track = getTrack();
+	if (track->f.compressionType == AF_COMPRESSION_IMA)
+		initIMACompressionParams();
+}
+
+void AIFFFile::initIMACompressionParams()
+{
+	Track *track = getTrack();
+
+	track->f.bytesPerPacket = 34 * track->f.channelCount;
+	track->f.framesPerPacket = 64;
+
+	AUpvlist pv = AUpvnew(1);
+	AUpvsetparam(pv, 0, _AF_IMA_ADPCM_TYPE);
+	AUpvsetvaltype(pv, 0, AU_PVTYPE_LONG);
+	long l = _AF_IMA_ADPCM_TYPE_QT;
+	AUpvsetval(pv, 0, &l);
+
+	track->f.compressionParams = pv;
+}
+
+// Read a Pascal-style string.
+bool AIFFFile::readPString(char s[256])
+{
+	uint8_t length;
+	if (m_fh->read(&length, 1) != 1)
+		return false;
+	if (m_fh->read(s, length) != static_cast<ssize_t>(length))
+		return false;
+	s[length] = '\0';
+	return true;
+}
+
+// Write a Pascal-style string.
+bool AIFFFile::writePString(const char *s)
+{
+	size_t length = strlen(s);
+	if (length > 255)
+		return false;
+	uint8_t sizeByte = static_cast<uint8_t>(length);
+	if (m_fh->write(&sizeByte, 1) != 1)
+		return false;
+	if (m_fh->write(s, length) != (ssize_t) length)
+		return false;
+	/*
+		Add a pad byte if the length of the Pascal-style string
+		(including the size byte) is odd.
+	*/
+	if ((length % 2) == 0)
+	{
+		uint8_t zero = 0;
+		if (m_fh->write(&zero, 1) != 1)
+			return false;
+	}
+	return true;
+}
