@@ -1,21 +1,21 @@
 /*
 	Audio File Library
-	Copyright (C) 2010, Michael Pruett <michael@68k.org>
+	Copyright (C) 2010-2012, Michael Pruett <michael@68k.org>
 
 	This library is free software; you can redistribute it and/or
-	modify it under the terms of the GNU Library General Public
+	modify it under the terms of the GNU Lesser General Public
 	License as published by the Free Software Foundation; either
-	version 2 of the License, or (at your option) any later version.
+	version 2.1 of the License, or (at your option) any later version.
 
 	This library is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	Library General Public License for more details.
+	Lesser General Public License for more details.
 
-	You should have received a copy of the GNU Library General Public
+	You should have received a copy of the GNU Lesser General Public
 	License along with this library; if not, write to the
-	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-	Boston, MA  02111-1307  USA.
+	Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+	Boston, MA  02110-1301  USA
 */
 
 #ifndef FILE_MODULE_H
@@ -25,6 +25,11 @@
 
 class FileModule : public Module
 {
+public:
+	virtual bool handlesSeeking() const { return false; }
+
+	virtual int bufferSize() const;
+
 protected:
 	enum Mode { Compress, Decompress };
 	FileModule(Mode, Track *, File *fh, bool canSeek);
@@ -34,7 +39,9 @@ protected:
 
 	ssize_t read(void *data, size_t nbytes);
 	ssize_t write(const void *data, size_t nbytes);
+	off_t seek(off_t offset);
 	off_t tell();
+	off_t length();
 
 private:
 	Mode m_mode;
@@ -42,6 +49,9 @@ private:
 protected:
 	Track *m_track;
 
+	void reportReadError(AFframecount framesRead, AFframecount framesRequested);
+	void reportWriteError(AFframecount framesWritten, AFframecount framesRequested);
+
 private:
 	File *m_fh;
 	bool m_canSeek;
