@@ -4,19 +4,19 @@
 	Copyright (C) 2010, Michael Pruett <michael@68k.org>
 
 	This library is free software; you can redistribute it and/or
-	modify it under the terms of the GNU Library General Public
+	modify it under the terms of the GNU Lesser General Public
 	License as published by the Free Software Foundation; either
-	version 2 of the License, or (at your option) any later version.
+	version 2.1 of the License, or (at your option) any later version.
 
 	This library is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	Library General Public License for more details.
+	Lesser General Public License for more details.
 
-	You should have received a copy of the GNU Library General Public
+	You should have received a copy of the GNU Lesser General Public
 	License along with this library; if not, write to the
-	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-	Boston, MA  02111-1307  USA.
+	Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+	Boston, MA  02110-1301  USA
 */
 
 #ifndef MODULE_H
@@ -75,9 +75,9 @@ public:
 
 	void setSink(Module *);
 	void setSource(Module *);
-	Chunk *inChunk() { return m_inChunk.get(); }
+	Chunk *inChunk() const { return m_inChunk.get(); }
 	void setInChunk(Chunk *chunk) { m_inChunk = chunk; }
-	Chunk *outChunk() { return m_outChunk.get(); }
+	Chunk *outChunk() const { return m_outChunk.get(); }
 	void setOutChunk(Chunk *chunk) { m_outChunk = chunk; }
 
 	virtual const char *name() const;
